import os

from piperider_cli.error import PipeRiderConnectorError
from . import DataSource
from .field import TextField, PathField, ListField

APPLICATION_DEFAULT_CREDENTIALS = os.path.join(os.path.expanduser('~'), '.config', 'gcloud',
                                               'application_default_credentials.json')
DEFAULT_GCP_CREDENTIALS = os.environ.get('GOOGLE_APPLICATION_CREDENTIALS', APPLICATION_DEFAULT_CREDENTIALS)
AUTH_METHOD_OAUTH = 'oauth'
AUTH_METHOD_OAUTH_SECRETS = 'oauth_secrets'
AUTH_METHOD_SERVICE_ACCOUNT = 'service-account'
AUTH_METHOD_SERVICE_ACCOUNT_JSON = 'service-account-json'


class BigQueryDataSource(DataSource):
    def __init__(self, name, **kwargs):
        super().__init__(name, 'bigquery', **kwargs)
        self.fields = [
            TextField('project', description='GCP Project ID'),
            ListField('method', description='Authentication Methods',
                      default=[AUTH_METHOD_OAUTH, AUTH_METHOD_SERVICE_ACCOUNT]),
            PathField('keyfile', description='The path of GCP Service Account Key File',
                      default=DEFAULT_GCP_CREDENTIALS if os.path.exists(DEFAULT_GCP_CREDENTIALS) else None,
                      ignore=lambda answers: AUTH_METHOD_SERVICE_ACCOUNT != answers['method']),
            TextField('dataset', description='The name of BigQuery DataSet'),
        ]

    def validate(self):
        reasons = []
        if self.type_name != 'bigquery':
            raise ValueError('type name should be bigquery')

        method = self.credential.get('method')
        if method not in [AUTH_METHOD_OAUTH, AUTH_METHOD_SERVICE_ACCOUNT,
                          AUTH_METHOD_SERVICE_ACCOUNT_JSON]:
            reasons.append(f'Unsupported authentication method: {method}')

        if method == AUTH_METHOD_OAUTH:
            if not os.path.exists(DEFAULT_GCP_CREDENTIALS):
                reasons.append(
                    f'Default Credential file {DEFAULT_GCP_CREDENTIALS} is not found.\n'
                    'Please execute the following command to generate it.\n'
                    '[green]'
                    'gcloud auth application-default login \\\n'
                    '--scopes=https://www.googleapis.com/auth/bigquery,\\\n'
                    'https://www.googleapis.com/auth/drive.readonly,\\\n'
                    'https://www.googleapis.com/auth/iam.test'
                    '[/green]')
        if method == AUTH_METHOD_SERVICE_ACCOUNT:
            if self.credential.get('keyfile') is None:
                reasons.append('keyfile is required when authentication method is service-account')
        if method == AUTH_METHOD_SERVICE_ACCOUNT_JSON:
            if self.credential.get('keyfile_json') is None:
                reasons.append('keyfile_json is required when authentication method is service-account-json')

        return reasons == [], reasons

    def to_database_url(self):
        from pybigquery.sqlalchemy_bigquery import BigQueryDialect
        BigQueryDialect.supports_statement_cache = True
        return f'bigquery://{self.credential["project"]}/{self.credential["dataset"]}'

    def engine_args(self):
        args = dict()
        if self.credential.get('method') == AUTH_METHOD_SERVICE_ACCOUNT:
            args['credentials_path'] = self.credential.get('keyfile')
        elif self.credential.get('method') == AUTH_METHOD_SERVICE_ACCOUNT_JSON:
            args['credentials_info'] = self.credential.get('keyfile_json', {})
        return args

    def verify_connector(self):
        try:
            import pybigquery

            # do nothing when everything is ok
            return None
        except Exception as e:
            return PipeRiderConnectorError(str(e), 'bigquery')



import os
import sys
import traceback
import typing as t

import sentry_sdk
from click.core import Command, Context
from rich.console import Console

from piperider_cli import event
from piperider_cli.error import PipeRiderError
from piperider_cli.guide import Guide

_enable_trackback: bool = os.environ.get('PIPERIDER_PRINT_TRACKBACK') == '1'

guide = Guide()


class TrackCommand(Command):

    def __init__(
        self,
        name: t.Optional[str],
        context_settings: t.Optional[t.Dict[str, t.Any]] = None,
        callback: t.Optional[t.Callable[..., t.Any]] = None,
        params: t.Any = None,
        help: t.Optional[str] = None,
        epilog: t.Optional[str] = None,
        short_help: t.Optional[str] = None,
        options_metavar: t.Optional[str] = "[OPTIONS]",
        add_help_option: bool = True,
        no_args_is_help: bool = False,
        hidden: bool = False,
        deprecated: bool = False,
    ) -> None:
        super(TrackCommand, self).__init__(name, context_settings, callback, params, help, epilog, short_help,
                                           options_metavar, add_help_option, no_args_is_help, hidden, deprecated)

    def _show_error_message(self, msg, params):
        console = Console()
        if params.get('debug'):
            console.print_exception(show_locals=True)
        else:
            console.print(f'[bold red]Error:[/bold red] {msg}')

    def _show_hint_message(self, hint):
        console = Console()
        console.print(f'[bold yellow]Hint[/bold yellow]:\n  {hint}')

    def invoke(self, ctx: Context) -> t.Any:
        status = False
        try:

            ret = super(TrackCommand, self).invoke(ctx)
            guide.show_tips(ctx.command.name)

            status = True
            return ret
        except Exception as e:
            if _enable_trackback:
                print(traceback.format_exc())
            self._show_error_message(e, ctx.params)
            if isinstance(e, PipeRiderError):
                if e.hint:
                    self._show_hint_message(e.hint)
            sentry_sdk.capture_exception(e)
            sentry_sdk.flush()
            sys.exit(1)
        finally:
            event.log_usage_event(ctx.command.name, ctx.params, status)
            event.flush_events(ctx.command.name)
